@@ -31,6 +31,7 @@
 
 #include "stri_stringi.h"
 #include "stri_container_utf16.h"
+#include "stri_container_utf8.h"
 #include "stri_container_usearch.h"
 #include <unicode/uregex.h>
 
@@ -42,7 +43,8 @@
  * @param pattern character vector
  * @param omit_na single logical value
  * @param opts_collator passed to stri__ucol_open(),
- * if \code{NA}, then \code{stri_detect_fixed_byte} is called
+ * if {NA}, then {stri_detect_fixed_byte} is called
+ *
  * @return character vector
  *
  * @version 0.3-1 (Bartek Tartanus, 2014-07-25)
@@ -51,23 +53,36 @@
  *                using std::vector<int> to avoid mem-leaks
  *
  * @version 0.3-1 (Marek Gagolewski, 2014-11-04)
- *    Issue #112: str_prepare_arg* retvals were not PROTECTed from gc
+ *    #112: str_prepare_arg* retvals were not PROTECTed from gc
  *
  * @version 0.3-1 (Marek Gagolewski, 2014-11-06)
  *    Added missing ucol_close
  *
  * @version 0.4-1 (Marek Gagolewski, 2014-12-04)
- *    FR #122: omit_na arg added
+ *    #122: omit_na arg added
  *
  * @version 1.0-3 (Marek Gagolewski, 2016-02-03)
- *    FR #216: `negate` arg added
+ *    #216: `negate` arg added
+ *
+ * @version 1.7.1 (Marek Gagolewski, 2021-06-17)
+ *    assure LENGTH(pattern) <= LENGTH(str)
  */
 SEXP stri_subset_coll(SEXP str, SEXP pattern, SEXP omit_na, SEXP negate, SEXP opts_collator)
 {
     bool negate_1 = stri__prepare_arg_logical_1_notNA(negate, "negate");
     bool omit_na1 = stri__prepare_arg_logical_1_notNA(omit_na, "omit_na");
-    PROTECT(str = stri_prepare_arg_string(str, "str"));
-    PROTECT(pattern = stri_prepare_arg_string(pattern, "pattern"));
+    PROTECT(str = stri__prepare_arg_string(str, "str"));
+    PROTECT(pattern = stri__prepare_arg_string(pattern, "pattern"));
+
+    if (LENGTH(str) > 0 && LENGTH(str) < LENGTH(pattern))
+        Rf_error(MSG__WARN_RECYCLING_RULE2);
+
+    R_len_t vectorize_length = stri__recycling_rule(true, 2, LENGTH(str), LENGTH(pattern));
+
+    if (vectorize_length == 0) {
+        UNPROTECT(2);
+        return Rf_allocVector(STRSXP, 0);
+    }
 
     // call stri__ucol_open after prepare_arg:
     // if prepare_arg had failed, we would have a mem leak
@@ -75,7 +90,6 @@ SEXP stri_subset_coll(SEXP str, SEXP pattern, SEXP omit_na, SEXP negate, SEXP op
     collator = stri__ucol_open(opts_collator);
 
     STRI__ERROR_HANDLER_BEGIN(2)
-    R_len_t vectorize_length = stri__recycling_rule(true, 2, LENGTH(str), LENGTH(pattern));
     StriContainerUTF16 str_cont(str, vectorize_length);
     StriContainerUStringSearch pattern_cont(pattern, vectorize_length, collator);  // collator is not owned by pattern_cont
 
@@ -90,7 +104,7 @@ SEXP stri_subset_coll(SEXP str, SEXP pattern, SEXP omit_na, SEXP negate, SEXP op
             i = pattern_cont.vectorize_next(i))
     {
         STRI__CONTINUE_ON_EMPTY_OR_NA_STR_PATTERN(str_cont, pattern_cont,
-    {if (omit_na1) which[i] = FALSE; else {
+        {if (omit_na1) which[i] = FALSE; else {
                 which[i] = NA_LOGICAL;
                 result_counter++;
             }
@@ -133,22 +147,33 @@ SEXP stri_subset_coll(SEXP str, SEXP pattern, SEXP omit_na, SEXP negate, SEXP op
  * @return character vector
  *
  * @version 1.0-3 (Marek Gagolewski, 2016-02-03)
- *   FR#124
+ *   #124
  *
  * @version 1.0-3 (Marek Gagolewski, 2016-02-03)
- *    FR #216: `negate` arg added
+ *   #216: `negate` arg added
+ *
+ * @version 1.7.1 (Marek Gagolewski, 2021-06-17)
+ *    assure LENGTH(pattern) and LENGTH(value) <= LENGTH(str)
  */
 SEXP stri_subset_coll_replacement(SEXP str, SEXP pattern, SEXP negate, SEXP opts_collator, SEXP value)
 {
     bool negate_1 = stri__prepare_arg_logical_1_notNA(negate, "negate");
-    PROTECT(str = stri_prepare_arg_string(str, "str"));
-    PROTECT(pattern = stri_prepare_arg_string_1(pattern, "pattern"));
-    PROTECT(value = stri_prepare_arg_string(value, "value"));
+    PROTECT(str = stri__prepare_arg_string(str, "str"));
+    PROTECT(pattern = stri__prepare_arg_string(pattern, "pattern"));
+    PROTECT(value = stri__prepare_arg_string(value, "value"));
 
-    int vectorize_length = LENGTH(str);
-    int value_length = LENGTH(value);
-    if (value_length == 0)
-        Rf_error(MSG__REPLACEMENT_ZERO);
+    // we are subsetting `str`, therefore recycling is slightly different here
+    if (LENGTH(value) == 0) Rf_error(MSG__REPLACEMENT_ZERO);
+    if (LENGTH(pattern) == 0) Rf_error(MSG__WARN_EMPTY_VECTOR);
+    if (LENGTH(str) == 0) {
+        UNPROTECT(3);
+        return Rf_allocVector(STRSXP, 0);
+    }
+    if (LENGTH(str) < LENGTH(pattern))  // for LENGTH(value), we emit warning later on
+        Rf_error(MSG__WARN_RECYCLING_RULE2);
+    if ((LENGTH(str) % LENGTH(pattern)) != 0)
+        Rf_warning(MSG__WARN_RECYCLING_RULE);
+    R_len_t vectorize_length = LENGTH(str);
 
     // call stri__ucol_open after prepare_arg:
     // if prepare_arg had failed, we would have a mem leak
@@ -156,33 +181,47 @@ SEXP stri_subset_coll_replacement(SEXP str, SEXP pattern, SEXP negate, SEXP opts
     collator = stri__ucol_open(opts_collator);
 
     STRI__ERROR_HANDLER_BEGIN(3)
+    R_len_t value_length = LENGTH(value);
+    StriContainerUTF8 value_cont(value, value_length);
     StriContainerUTF16 str_cont(str, vectorize_length);
     StriContainerUStringSearch pattern_cont(pattern, vectorize_length, collator);  // collator is not owned by pattern_cont
-    StriContainerUTF8 value_cont(value, value_length);
 
     SEXP ret;
     STRI__PROTECT(ret = Rf_allocVector(STRSXP, vectorize_length));
 
-    R_len_t k = 0;
-    for (R_len_t i = str_cont.vectorize_init();
-            i != str_cont.vectorize_end();
-            i = str_cont.vectorize_next(i))
+    std::vector<int> detected(vectorize_length, 0);
+    for (R_len_t i = pattern_cont.vectorize_init();
+            i != pattern_cont.vectorize_end();
+            i = pattern_cont.vectorize_next(i))
     {
+        if (pattern_cont.isNA(i)) {
+            // behave like `[<-`
+            detected[i] = false;
+            continue;
+        }
         STRI__CONTINUE_ON_EMPTY_OR_NA_STR_PATTERN(str_cont, pattern_cont,
-        {SET_STRING_ELT(ret, i, NA_STRING);},
-        {SET_STRING_ELT(ret, i, (negate_1)?value_cont.toR((k++)%value_length):str_cont.toR(i));})
+        {detected[i] = NA_INTEGER;},
+        {detected[i] = negate_1;})
 
         UStringSearch *matcher = pattern_cont.getMatcher(i, str_cont.get(i));
         usearch_reset(matcher);
         UErrorCode status = U_ZERO_ERROR;
-        if (((int)usearch_first(matcher, &status) != USEARCH_DONE && !negate_1)
-                || (usearch_first(matcher, &status) == USEARCH_DONE && negate_1))
-            SET_STRING_ELT(ret, i, value_cont.toR((k++)%value_length));
-        else
-            SET_STRING_ELT(ret, i, str_cont.toR(i));
+        detected[i] = (((int)usearch_first(matcher, &status) != USEARCH_DONE && !negate_1)
+                || (usearch_first(matcher, &status) == USEARCH_DONE && negate_1));
         STRI__CHECKICUSTATUS_THROW(status, {/* do nothing special on err */})
     }
 
+    R_len_t k = 0;  // we must traverse `str_cont` in order now
+    for (R_len_t i = 0; i<vectorize_length; ++i) {
+        if (detected[i] == NA_INTEGER)
+            SET_STRING_ELT(ret, i, NA_STRING);
+        else if (detected[i] == 0)
+            SET_STRING_ELT(ret, i, str_cont.toR(i));
+        else
+            SET_STRING_ELT(ret, i, value_cont.toR((k++)%value_length));
+    }
+    if ((k % value_length) != 0) Rf_warning(MSG_REPLACEMENT_MULTIPLE);
+
     if (collator) {
         ucol_close(collator);
         collator = NULL;
